@@ -13,6 +13,7 @@
 import os
 import sys
 import glob
+import codecs
 import random
 import warnings
 import _thread
@@ -61,7 +62,7 @@ def _fopen(fname, mode):
         2) "$cmd |" which means pipe.stdout
     """
     if mode not in ["w", "r", "wb", "rb"]:
-        raise ValueError("Unknown open mode: {mode}".format(mode=mode))
+        raise ValueError(f"Unknown open mode: {mode}")
     if not fname:
         return None
     fname = fname.rstrip()
@@ -75,9 +76,11 @@ def _fopen(fname, mode):
         return pin if mode == "rb" else TextIOWrapper(pin)
     else:
         if mode in ["r", "rb"] and not os.path.exists(fname):
-            raise FileNotFoundError(
-                "Could not find common file: {}".format(fname))
-        return open(fname, mode)
+            raise FileNotFoundError(f"Could not find common file: {fname}")
+        if mode in ["r", "w"]:
+            return codecs.open(fname, mode, encoding="utf-8")
+        else:
+            return open(fname, mode)
 
 
 def _fclose(fname, fd):
@@ -144,12 +147,17 @@ def parse_scps(scp_path,
 
 class Reader(object):
     """
-        Base class for sequential/random accessing, to be implemented
+    Base class for sequential/random accessing, to be implemented
     """
-    def __init__(self, scp_path, value_processor=lambda x: x, num_tokens=2):
+    def __init__(self,
+                 scp_path,
+                 value_processor=lambda x: x,
+                 num_tokens=2,
+                 restrict=True):
         self.index_dict = parse_scps(scp_path,
                                      value_processor=value_processor,
-                                     num_tokens=num_tokens)
+                                     num_tokens=num_tokens,
+                                     restrict=restrict)
         self.index_keys = list(self.index_dict.keys())
 
     # return values
@@ -188,7 +196,7 @@ class Reader(object):
 
 class Writer(object):
     """
-        Base class, to be implemented
+    Base class, to be implemented
     """
     def __init__(self, ark_path, scp_path=None):
         self.scp_path = scp_path
@@ -196,7 +204,7 @@ class Writer(object):
         # if dump ark to output, then ignore scp
         if ark_path == "-" and scp_path:
             warnings.warn(
-                "Ignore .scp output discriptor cause dump archives to stdout")
+                "Ignore .scp output descriptor cause dump archives to stdout")
             self.scp_path = None
         # "wb" is important
         self.ark_file = _fopen(self.ark_path, "wb")
@@ -215,7 +223,7 @@ class Writer(object):
 
 class SequentialReader(object):
     """
-        Base class for sequential reader(only for .ark/.egs)
+    Base class for sequential reader (only for .ark/.egs)
     """
     def __init__(self, ark_or_pipe):
         self.ark_or_pipe = ark_or_pipe
@@ -226,7 +234,7 @@ class SequentialReader(object):
 
 class ScriptReader(Reader):
     """
-        Reader for kaldi's scripts(for BaseFloat matrix)
+    Reader for kaldi's scripts (for BaseFloat matrix)
     """
     def __init__(self, ark_scp):
         self.fmgr = dict()
@@ -261,7 +269,7 @@ class ScriptReader(Reader):
 
 class ArchiveReader(SequentialReader):
     """
-        Sequential Reader for Kalid's archive(.ark) object
+    Sequential Reader for Kalid's archive (.ark) object
     """
     def __init__(self, ark_or_pipe):
         super(ArchiveReader, self).__init__(ark_or_pipe)
@@ -274,7 +282,7 @@ class ArchiveReader(SequentialReader):
 
 class Nnet3EgsReader(SequentialReader):
     """
-        Sequential Reader for Kalid's nnet3 .egs object
+    Sequential Reader for Kalid's nnet3 .egs object
     """
     def __init__(self, ark_or_pipe):
         super(Nnet3EgsReader, self).__init__(ark_or_pipe)
@@ -287,7 +295,7 @@ class Nnet3EgsReader(SequentialReader):
 
 class AlignArchiveReader(SequentialReader):
     """
-        Reader for kaldi's alignment archives
+    Reader for kaldi's alignment archives
     """
     def __init__(self, ark_or_pipe):
         super(AlignArchiveReader, self).__init__(ark_or_pipe)
@@ -300,7 +308,7 @@ class AlignArchiveReader(SequentialReader):
 
 class AlignScriptReader(ScriptReader):
     """
-        Reader for kaldi's scripts(for int32 vector, such as alignments)
+    Reader for kaldi's scripts (for int32 vector, such as alignments)
     """
     def __init__(self, ark_scp):
         super(AlignScriptReader, self).__init__(ark_scp)
@@ -314,7 +322,7 @@ class AlignScriptReader(ScriptReader):
 
 class ArchiveWriter(Writer):
     """
-        Writer for kaldi's archive && scripts (for BaseFloat matrix)
+    Writer for kaldi's archive && scripts (for BaseFloat matrix)
     """
     def __init__(self, ark_path, scp_path=None):
         super(ArchiveWriter, self).__init__(ark_path, scp_path)
